
#include <msp430x12x2.h>
#include "nRF24L01.h"
#include "const.h"

#define CSN_TIME      2
#define CE_HIGH_TIME  100

unsigned char RX_ADDRESS_P0[5]  = {5,6,7,8,9};
unsigned char RX_ADDRESS_P1[5]  = {0,1,2,3,4};
unsigned char TX_ADDRESS[5]     = {5,6,7,8,9};
unsigned char ADDRESS[5];

unsigned char status, check;

void Delay (unsigned long a) { while (--a!=0); }


// Chip Select -> high
void CSN_HIGH (void)  { P3OUT |= BIT0; }
// Chip Select -> low
void CSN_LOW (void)   { Delay(CSN_TIME); P3OUT &= ~BIT0; }

// Chip enable High
void CE_HIGH(void)    { P3OUT |= BIT6;  Delay(CE_HIGH_TIME); }
// Chip enable low
void CE_LOW(void)     { P3OUT &= ~BIT6; }


// Init SPI0 Interface
void SPI0Init (void) {

  // Enable USART1 SPI mode
  ME2 = 0x01;		
  // 8-bit SPI Master **SWRST**
  U0CTL = 0x16;	
  // ACLK, 3-pin mode, clock idle low, data valid on rising edge, UCLK delayed
  U0TCTL =  0x12;	
  // SMCLK, 3-pin mode, clock idle low, data valid on rising edge, UCLK delayed
  // U0TCTL =  0x22;	

  // Baud Rate
  UBR00 = 0x02;			// 0x02: UCLK/2 (16 kHz)
  UBR10 = 0x00;			// -"-
  U0MCTL = 0x00;		// no modulation

  // Pin selection
  P3SEL |= 0x0e;		// P3.1-3 SPI option select
  P3DIR |= 0x0b;		// P3.0 output direction

  // Clear Rx flag
  IFG2 &= ~URXIFG0;	

}

// Transmit byte via SPI0 chanel
unsigned char  SPI_SendByte(unsigned char data) {

  // variable
  unsigned char result;

  // Write byte
  TXBUF0 = data & 0xFF;

  // USART0 RX buffer ready
  while ((IFG2 & URXIFG0) == 0);
  
  // Clear Rx flag
  IFG2 &= ~URXIFG0;	

  // Read byte
  result = RXBUF0;

  // return read byte
  return result;

}

unsigned char SPI_Send_command_with_ADDR (unsigned char cmd, unsigned char addr, unsigned char data_byte)
{
  unsigned char temp,command = 0;
  int k,j;
  temp=0;
  command = (cmd << 5) | addr;
  CSN_LOW();
  if (cmd == R_REGISTER)
  {
    if (addr == RX_ADDR_P0 || addr == RX_ADDR_P1 || addr == TX_ADDR)
    {

      status=SPI_SendByte(command);
      for (k=0;k!=5;k++)
      {
        ADDRESS[k]=SPI_SendByte(NOP);
      }
      CSN_HIGH();
      return status;
    }
    else
    {
      status=SPI_SendByte(command);
      temp=SPI_SendByte(NOP);
      CSN_HIGH();
      return temp;
    }
  }
  if (cmd == W_REGISTER)
  {
    if (addr == RX_ADDR_P0)
    {
      status=SPI_SendByte(command);
      for (j=0;j!=5;j++)
        {
          temp=RX_ADDRESS_P0[j];
          SPI_SendByte(temp);
        }
      CSN_HIGH();
      return status;
    }

    if (addr == RX_ADDR_P1)
    {
      status=SPI_SendByte(command);
      for (j=0;j!=5;j++)
        {
          temp=RX_ADDRESS_P1[j];
          SPI_SendByte(temp);
        }
      CSN_HIGH();
      return status;
    }

    if (addr == TX_ADDR)
    {
      status=SPI_SendByte(command);
      for (j=0;j!=5;j++)
        {
          temp=TX_ADDRESS[j];
          SPI_SendByte(temp);
        }
      CSN_HIGH();
      return status;
    }


    else
    {
      temp=SPI_SendByte(command);
      SPI_SendByte(data_byte);
      CSN_HIGH();
      return temp;
    }
  }

  return 1;
}

unsigned char SPI_Send_command_without_ADDR (unsigned char cmd, unsigned char data_byte)
{
  unsigned char temp = 0;
  CSN_LOW();
  if (cmd == R_RX_PAYLOAD)
  {
    status=SPI_SendByte(cmd);
    temp=SPI_SendByte(NOP);
    CSN_HIGH();
    return temp;
  }
  if (cmd == W_TX_PAYLOAD)
  {
    status=SPI_SendByte(cmd);
    SPI_SendByte(data_byte);
    CSN_HIGH();
    return status;
  }
  status = SPI_SendByte(cmd);
  CSN_HIGH();
  return status;

}

// Setting for nRF chip
void nRFSetting(void) {

  // Discard transmision
  CE_LOW();

  //Write CONFIG register (addres - 0x00)
  //00001010 - CRC enable, power-up, RX
  status = SPI_Send_command_with_ADDR(W_REGISTER, CONFIG_REG_ADDR, 0x2B);
  // read
  // check = SPI_Send_command_with_ADDR(R_REGISTER, CONFIG_REG_ADDR, NOP);

  //Write RX_ADDR_P0 register -> Set receive address data Pipe0 -> address in RX_ADDRESS_P0 array
  status = SPI_Send_command_with_ADDR(W_REGISTER, RX_ADDR_P0, NOP);
  // read
  // status = SPI_Send_command_with_ADDR(R_REGISTER, RX_ADDR_P0, NOP);

  //Write RX_ADDR_P1 register -> Set receive address data Pipe1 -> address in RX_ADDRESS_P1 array
  status = SPI_Send_command_with_ADDR(W_REGISTER, RX_ADDR_P1, NOP);
  // read
  // status = SPI_Send_command_with_ADDR(R_REGISTER,RX_ADDR_P1, NOP);

  //Write TX_ADDR register -> Transmit address. Used for a PTX device only. Address in TX_ADDRESS array
  status = SPI_Send_command_with_ADDR(W_REGISTER, TX_ADDR, NOP);
  // read
  // status = SPI_Send_command_with_ADDR(R_REGISTER, TX_ADDR, NOP);

  //Write RX_PW_P0 register -> Set number of bytes in RX payload in data pipe0 -> 1 byte
  status = SPI_Send_command_with_ADDR(W_REGISTER, RX_PW_P0, 1);
  // read
  // dummy = SPI_Send_command_with_ADDR(R_REGISTER, RX_PW_P0, NOP);

  //Write RX_PW_P1 register -> Set number of bytes in RX payload in data pipe1 -> 1 byte
  status = SPI_Send_command_with_ADDR(W_REGISTER, RX_PW_P1, 1);
  // read
  // dummy = SPI_Send_command_with_ADDR(R_REGISTER, RX_PW_P1, NOP);

}

