//system.c
#include "system.h"
#include <msp430x12x2.h>
#include "bits.h"
#include "const.h"

// just simple delay
void sDelay (unsigned long a) { while (--a!=0); }


// System initialization
void SysInit(void) {

  // IRQ as input
  P2DIR &= ~BIT4;
   
  // BUT as input
  P2DIR &= ~BIT3;
   
  // LED as output
  P3DIR |= BIT7;

  // CE as outpit
  P3DIR |= BIT6;
}


// Init frequency
void FreqInit_DCO(void) {

  // Watchdog
  WDTCTL = WDTPW + WDTHOLD;   // Stop watchdog timer

  // DCO Control Register
  // register:
  // MODx   4-0
  // DCOx   7-5
  // set:
  // MODx = 0, DCOx = 2
  // description:
  // modulation off, DCOx and RSELx ~1MHz
   DCOCTL = 0xE0;

  // Basic Clock System Control Register 1
  // register:
  // RSELx  2-0
  // XT5V   3
  // DIVAx  5-4
  // XTS    6
  // XT20FF 7
  // set:
  // RSELx = 5, DIVAx = 0, XTS = 0, XT20FF = 1
  // description:
  // no divider for ACLK, set Lower mode, XT2 off
  BCSCTL1 = 0x84;

  // Basic Clock System Control Register 2
  // register:
  // DCOR   0
  // DIVSx  1-2
  // SELS   3
  // DIVMx  4-5
  // SELMx  6-7
  // set:
  // DCOR = 0, DIVSx = 0, SELS = 0, DIVMx = 0, SELMx = 0
  // description:
  // internal resistor for DCO, no divider for MCLK and SCLK, MCLK and SCLK source -> DCOCLK
  BCSCTL2 = 0x00;

}

void FreqInit_LFXT1(void) {

  // LFXT1 -> External oscilator 32kHz

  // Watchdog
  WDTCTL = WDTPW + WDTHOLD;   // Stop watchdog timer

  // DCO Control Register
  // register:
  // MODx   4-0
  // DCOx   7-5
  // set:
  // MODx = 0, DCOx = 0
  // description:
  // modulation off
   DCOCTL = 0x00;

  // Basic Clock System Control Register 1
  // register:
  // RSELx  2-0
  // XT5V   3
  // DIVAx  5-4
  // XTS    6
  // XT20FF 7
  // set:
  // RSELx = 0, DIVAx = 0, XTS = 0, XT20FF = 1
  // description:
  // no divider for ACLK, set Lower mode, XT2 off
  BCSCTL1 = 0x80;

  // Basic Clock System Control Register 2
  // register:
  // DCOR   0
  // DIVSx  1-2
  // SELS   3
  // DIVMx  4-5
  // SELMx  6-7
  // set:
  // DCOR = 0, DIVSx = 0, SELS = 1, DIVMx = 0, SELMx = 3
  // description:
  // internal resistor for DCO, no divider for MCLK and SCLK, MCLK and SCLK source -> LFXT1CLK(32KHz)
  BCSCTL2 = 0xC8;

}


// Init Interrupts
void InitInterrupts(void) {

  // Ports P2.3, P2.4 as input
  P2DIR   &= ~0x18;

  // Set high-to-low transition for P2.3, P2.4
  P2IES   |= 0x18;
  
  // Enable interupts for P2.3, P2.4
  P2IE    |= 0x18;

  // Clear all interrupts flag
  P1IFG = 0x0;
  P2IFG = 0x0;

}
